// file      : examples/cxx/tree/polymorphism/driver.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : not copyrighted - public domain

#include <memory>   // std::auto_ptr
#include <iostream>
#include <string>

#include <xercesc/dom/DOM.hpp>

#include "x3d-3.0.hxx"

// The following string class keeps us sane when working with Xerces.
// Include it after the generated header in order to get only char or
// wchar_t version depending on how you compiled your schemas.
//
#include <xsd/cxx/xml/string.hxx>

using std::cout;
using std::cerr;
using std::endl;
using std::auto_ptr;
using namespace xercesc;

int printX3DNodeData( DOMNode* current )
{
	xml_schema::type& t ( *reinterpret_cast<xml_schema::type*> ( current->getUserData (xml_schema::tree_node_key)));
	x3d::Transform& current_transform (dynamic_cast<x3d::Transform&> (t));
	cout << "Transform" << endl;
	cout << "containerField: " << current_transform.containerField() << endl;
	cout << "bboxCenter: " << current_transform.bboxCenter() << endl;
	cout << "bboxSize: " << current_transform.bboxSize() << endl;
	cout << "center: " << current_transform.center() << endl;
	cout << "rotation: " << current_transform.rotation() << endl;
	cout << "scale: " << current_transform.scale() << endl;
	cout << "scaleOrientation: " << current_transform.scaleOrientation() << endl;
	cout << "translation: " << current_transform.translation() << endl;
	return 0;
}

int getChildren( DOMNode* current, int indent = 0 )
{
  try
  {
	for( DOMNode* cur = current->getFirstChild();
		cur != 0; cur = cur->getNextSibling() )
	{
		for( int i=0;i<indent;i++ )
			cout << " ";
		cout << XMLString::transcode( cur->getNodeName() ) << endl;
		switch( cur->getNodeType() )
		{
			case DOMNode::ELEMENT_NODE:
			{
				if( cur->hasAttributes() )
				{
					DOMNamedNodeMap* attributes = cur->getAttributes();
					if( attributes )
					{
						for( unsigned int i=0;i<attributes->getLength();i++ )
						{
							for( int j=0;j<indent+2;j++ )
								cout << " ";
							DOMNode* attr = attributes->item(i);
							cout << XMLString::transcode( attr->getNodeName() );
							if( attr->getNodeValue() )
							{
								cout << " " << XMLString::transcode( attr->getNodeValue() ) << endl;
							}
						}
					}
				}
				if( XMLString::transcode( cur->getNodeName() ) == std::string( "Transform" ) )
				{
					if( cur->hasAttributes() )
					{
						printX3DNodeData( cur );
					}
				}
				break;
			}
		case DOMNode::ATTRIBUTE_NODE:
		case DOMNode::TEXT_NODE:
		case DOMNode::CDATA_SECTION_NODE:
		case DOMNode::ENTITY_REFERENCE_NODE:
		case DOMNode::ENTITY_NODE:
		case DOMNode::PROCESSING_INSTRUCTION_NODE:
		case DOMNode::COMMENT_NODE:
		case DOMNode::DOCUMENT_NODE:
		case DOMNode::DOCUMENT_TYPE_NODE:
		case DOMNode::DOCUMENT_FRAGMENT_NODE:
		case DOMNode::NOTATION_NODE :
		default:
			break;
		}
		cout << endl;
		if( cur->hasChildNodes() )
			getChildren( cur, indent + 2 );
	}
  }
  catch (const xml_schema::exception& e)
  {
    cerr << "HELP ME    " << e << endl;
    return 1;
  }
  return 0;
}

int main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " x3d::X3D.xml" << endl;
    return 1;
  }

  XMLPlatformUtils::Initialize ();

  try
  {
    xml_schema::properties props;

/*    char cwd[1024];
    if (getcwd (cwd, 1024) == 0)
    {
      cerr << "cwd is longer than 1024 bytes" << endl;
      return 1;
    }
*/
/*    props.schema_location (
      "http://www.w3.org/2001/XMLSchema-instance",   // namespace
      "file://" + std::string (cwd) + "/x3d-3.0.xsd");
*/
    props.no_namespace_schema_location (
	"http://www.web3d.org/specifications/x3d-3.0.xsd" );
//      "file://" + std::string (cwd) + "/x3d-3.0.xsd");


    auto_ptr<x3d::X3D> inst (x3d::X3D_ (argv[1], xml_schema::flags::keep_dom |
        xml_schema::flags::dont_initialize, props ));

    x3d::X3D copy (*inst); // Dynamic types are preserved in copies.

    {
      namespace xml = xsd::cxx::xml;

      DOMNode* root (copy._node ());

      getChildren( root );

    }
  }
  catch (const xml_schema::exception& e)
  {
    cerr << e << endl;
    return 1;
  }

  XMLPlatformUtils::Terminate ();

}
